@@ -1,4 +1,4 @@
-/* Copyright (c) 2020 Yoran Heling
+/* Copyright (c) 2020-2022 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -239,44 +239,20 @@ static void parse_qarg(struct parse_ctx *ctx) {
 }
 
 
-/* A "proper" directive would start with an identifier, but some modules
-   (e.g. http_core's 'types' directive) have special syntax. So let's just allow
-   any unquoted string that doesn't interfere with the rest of the syntax. */
-static char *parse_directive_name(struct parse_ctx *ctx) {
+// Parse either a quoted or unquoted string
+static char *parse_arg(struct parse_ctx *ctx) {
     size_t start = ctx->pos;
-    char *str;
-    while(1) {
-        switch(ctx->c) {
-            case 0:
-                die(ctx->ctx, "Unexpected EOF");
-            case '}':
-            case '\\':
-            case '$':
-            case '@':
-                die(ctx->ctx, "Unexpected character in directive name: '%c'", ctx->c);
-            case ' ':
-            case '\t':
-            case '\r':
-            case '\n':
-            case '#':
-            case ';':
-            case '{':
-                str = malloc(ctx->pos-start+1);
-                memcpy(str, ctx->buf+start, ctx->pos-start);
-                str[ctx->pos-start] = 0;
-                return str;
-            default:
-                parse_take(ctx);
-        }
-    }
-    return NULL;
+    (ctx->c == '"' || ctx->c == '\'' ? parse_qarg : parse_unqarg)(ctx);
+    char *str = malloc(ctx->pos-start+1);
+    memcpy(str, ctx->buf+start, ctx->pos-start);
+    str[ctx->pos-start] = 0;
+    return str;
 }
 
 
 static struct cfg_directive *parse_block(struct parse_ctx *ctx) {
     struct cfg_directive *head = NULL, *cur = NULL, *tmp;
     struct cfg_arg **arg;
-    size_t arg_start;
 
 directive_start:
     parse_ws(ctx);
@@ -287,7 +263,7 @@ directive_start:
     *(head ? &cur->next : &head) = tmp;
     cur = tmp;
 
-    cur->name = parse_directive_name(ctx);
+    cur->name = parse_arg(ctx);
     cur->ctx = *ctx->ctx;
     arg = &cur->args;
 
@@ -309,13 +285,7 @@ directive_arg:
         default:
             *arg = calloc(1, sizeof(struct cfg_arg));
             (*arg)->ctx = *ctx->ctx;
-
-            arg_start = ctx->pos;
-            (ctx->c == '"' || ctx->c == '\'' ? parse_qarg : parse_unqarg)(ctx);
-            (*arg)->data = malloc(ctx->pos-arg_start+1);
-            memcpy((*arg)->data, ctx->buf+arg_start, ctx->pos-arg_start);
-            (*arg)->data[ctx->pos-arg_start] = 0;
-
+            (*arg)->data = parse_arg(ctx);
             arg = &(*arg)->next;
             goto directive_arg;
     }
@@ -351,21 +321,21 @@ static struct cfg_directive *parse_file(struct ctx *ctx) {
  *   FORMATTING
  */
 
-void write_str(FILE *f, const char *s) {
+static void write_str(FILE *f, const char *s) {
     if(fputs(s, f) < 0) {
         fprintf(stderr, "Error writing to output: %s", strerror(errno));
         exit(1);
     }
 }
 
-void write_indent(FILE *f, int lvl) {
+static void write_indent(FILE *f, int lvl) {
     int i;
     for(i=0; i<lvl; i++)
         write_str(f, "    ");
 }
 
 
-void write_directive(FILE *f, int lvl, struct cfg_directive *d) {
+static void write_directive(FILE *f, int lvl, struct cfg_directive *d) {
     struct cfg_arg *arg = d->args;
     struct cfg_directive *body = d->body;
 
@@ -760,7 +730,7 @@ static int proc_if_infix(struct proc_ctx *ctx, struct ctx *if_ctx, char *a, char
         die(if_ctx, "Invalid regular expression: %s", buf);
     }
 
-    r = regexec(&reg, a, sizeof(match), match, 0);
+    r = regexec(&reg, a, sizeof(match)/sizeof(*match), match, 0);
     if(r != 0 && r != REG_NOMATCH) {
         regerror(r, &reg, buf, sizeof(buf));
         die(if_ctx, "Error executing regular expression: %s", buf);
